#coding: utf-8

import asyncore
import socket

from log import logger
from protocol import parse_graphite_line, format_info_tsdb

class Sender(asyncore.dispatcher, object):
    def __init__(self, remote, receiver):
        super(Sender, self).__init__()
        self.receiver = receiver
        self.remote = remote

        # Здесь обратная ссылка обусловлена необходимостью
        # закрытия сокета на стороне принимающего обработчика,
        # о которой мы не можем сигнализировать более простым способом.
        receiver.sender = self
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connect(self.remote)
        self.connected = False

    def handle_close(self):
        '''
        Обработчик нормального завершения соединения с TSD.
        '''
        logger.info('Connection with %s:%s closed' % self.remote)
        self.close()
        if self.receiver:
            # Если мы закрыли соединение, то нет и смысла что-то получать от клиента
            self.receiver.close()

    def handle_connect(self):
        '''
        Обработчик успешного соединения с TSD.
        '''
        logger.info('Connected to %s:%s' % self.remote)

    def writeable(self):
        # Пока есть ещё необработанные данные от клиента, мы будем писать!
        return len(self.receiver.buffer) > 0

    def handle_write(self):
        # Обрабатываем за раз все строки в буфере.
        # TODO: сделать порционную обработку, чтобы процессы
        # с большими объемами данных не блокировали остальных.
        while True:
            try:
                lf_pos = self.receiver.buffer.index('\n')
            except ValueError:
                break
            line = self.receiver.buffer[:lf_pos]
            try:
                info = parse_graphite_line(line)                
                self.send('%s\n' % format_info_tsdb(info))
            except ValueError:
                logger.error('Malformed line: %s' % line)
            self.receiver.buffer = self.receiver.buffer[lf_pos+1:]


class Receiver(asyncore.dispatcher, object):
    def __init__(self, fd):
        asyncore.dispatcher.__init__(self, fd)
        self.buffer = ''
        self.sender = None
        # Определяем IP и исходящий порт подключенного к нам клиента
        self.peer = self.socket.getpeername()

    def handle_read(self):
        read_bytes = self.recv(512)
        self.buffer += read_bytes

    def handle_close(self):
        logger.info('Connection with %s:%s closed' % self.peer)
        # Даже если мы разрываем соединение, необходимо дообработать все данные.
        self.handle_write()
        self.close()
        if self.sender:
            self.sender.close()


class Converter(asyncore.dispatcher, object):
    '''
    Класс, отвечающий за прием входящих соединений и создание дополнительных
    обработчиков для потоков данных. Так как каждый объект dispatcher способен
    обрабатывать события только от одного сокета, необходимо создать два дополнительных
    объекта, каждый из которых будет обрабатывать события своего конца соединения.
    '''

    def __init__(self, endpoint, remote, backlog=10):
        super(Converter, self).__init__()
        self.sender = None
        self.remote_addr = remote

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.set_reuse_addr()
        self.bind(endpoint)
        self.listen(backlog)
        logger.info('Listening on %s:%s...' % self.socket.getsockname())

    def handle_accept(self):
        socket_pair = self.accept()

        if socket_pair is not None:
            remote_sock, addr = socket_pair
            logger.info('Connection started with %s:%s' % addr)
            self.sender = Sender(self.remote_addr, Receiver(remote_sock))
