#!/usr/bin/env python
#coding: utf-8

import asyncore
import parse
import sys

from optparse import OptionParser
from protocol import parse_graphite_line, format_info_tsdb
from dispatchers import Converter
from log import setup_logging, logger


def main():
    bind_endpoint = None
    remote_endpoint = None

    setup_logging()
    parser = OptionParser()
    parser.add_option('-b', '--bind', dest='bind_endpoint', default='localhost:2003',
                      help='Bind at specified address and port (default: "localhost:2003").',
                      metavar="<host:port>")
    parser.add_option('-r', '--remote', dest='remote_endpoint', default='localhost:4242',
                      help='Connect to remote TSDB server (default: "localhost:4242").',
                      metavar="<host:port>")
    parser.add_option('-t', '--test', dest='test_mode', action='store_true',
                      help='Test patterns.')

    (options, args) = parser.parse_args()
    try:
        (host, port) = options.bind_endpoint.split(':')
        bind_endpoint = (host, int(port))
    except ValueError:
        logger.error('Invalid bind point: {0}'.format(options.bind_endpoint))

    try:
        (host, port) = options.remote_endpoint.split(':')
        remote_endpoint = (host, int(port))
    except ValueError:
        logger.error('Invalid TSDB endpoint: {0}'.format(options.bind_endpoint))

    if options.test_mode:
        # Режим тестирования для быстрой отладки паттернов переформатирования
        while True:
            line = sys.stdin.readline().strip()
            if not line:
                break

            try:
                info = parse_graphite_line(line)
                print format_info_tsdb(info)
            except ValueError:
                print 'Malformed line: {0}'.format(line)
    else:
        # Регистрируем обработчик входящих соединений
        converter = Converter(bind_endpoint, remote_endpoint)
        try:
            asyncore.loop()
        except KeyboardInterrupt:
            pass
